package physbeans.func;

import java.io.*;
import physbeans.math.*;

/**
 * bean for basic algebra operations.<br>
 * combines up to 4 operands using three basic operations:
 *         ((x op y) op z) op w<br>
 * other associativity schemes can be used as well.<br>
 * provides basic operations +, -, *, /, %, ^
 */
public class ArithmeticFunction extends GenericNdFunction implements Serializable {
  
  // implemented functions
  public static final int PLUS = 0;
  public static final int MINUS = 1;
  public static final int MULT = 2;
  public static final int DIV = 3;
  public static final int MOD = 4;
  public static final int POWER = 5;
  
  protected int firstOp = PLUS;
  protected int secondOp = PLUS;
  protected int thirdOp = PLUS;
  
  /** type of associativity */
  protected int associativity;
  
  /**
   * constructs a standard function
   */
  public ArithmeticFunction() {
    super(4,1);
    associativity = 0;
  }
  
  /**
   * combine two operands
   */
  protected double applyOp(double o1, double o2, int type) {
    double result = 0.0;
    switch (type) {
      case PLUS :
        result = o1 + o2;
        break;
      case MINUS :
        result = o1 - o2;
        break;
      case MULT :
        result = o1 * o2;
        break;
      case DIV :
        result = o1 / o2;
        break;
      case POWER :
        result = Math.pow(o1, o2);
        break;
      case MOD :
        // round doubles and use long arithmetic
        result = (double) (Math.round(o1) % Math.round(o2));
        break;
      default : // throw exception...
    }
    return result;
  }
  
  /**
   * computes the output value according to operator types<br>
   */
  protected DVector computeFunction(DVector in) {
    double x = in.get(0);
    double y = in.get(1);
    double z = in.get(2);
    double w = in.get(3);
    
    double t1, t2, res;
    switch (associativity) {
      case 0: // ((x op y) op z) op w
        t1  = applyOp(x, y, firstOp);
        t1  = applyOp(t1, z, secondOp);
        res = applyOp(t1, w, thirdOp);
        break;
      case 1:  // (x op y) op (z op w)
        t1  = applyOp(x, y, firstOp);
        t2  = applyOp(z, w, thirdOp);
        res = applyOp(t1, t2, secondOp);
        break;
      case 2:    // (x op (y op z)) op w
        t1  = applyOp(y, z, secondOp);
        t1  = applyOp(x, t1, firstOp);
        res = applyOp(t1, w, thirdOp);
        break;
      case 3: // x op ((y op z) op w)
        t1  = applyOp(y, z, secondOp);
        t1  = applyOp(t1, w, thirdOp);
        res = applyOp(x, t1, firstOp);
        break;
      case 4: // x op (y op (z op w))
        t1  = applyOp(z, w, thirdOp);
        t1  = applyOp(y, t1, secondOp);
        res = applyOp(x, t1, firstOp);
        break;
      default:
        throw new IllegalArgumentException("invalid associativity type");
    }
    
    return new DVector(res);
  }
  
  /**
   * get type of first operator
   */
  public int getFirstOp() {
    return firstOp;
  }
  
  /**
   * set type of first operator
   */
  public void setFirstOp(int op) {
    firstOp = op;
    trigger();
  }
  
  /**
   * get type of second operator
   */
  public int getSecondOp() {
    return secondOp;
  }
  
  /**
   * set type of second operator
   */
  public void setSecondOp(int op) {
    secondOp = op;
    trigger();
  }
  
  /**
   * get type of third operator
   */
  public int getThirdOp() {
    return thirdOp;
  }
  
  /**
   * set type of third operator
   */
  public void setThirdOp(int op) {
    thirdOp = op;
    trigger();
  }
  
  /**
   * Gets type of associativity.
   * Associativity is given by order of the three op's, so by a permutation.
   * Since the two permutations 312 and 132 give the same associativity,
   *    there are 5 different schemes:
   *      scheme                      ops    nr
   *    ((a op b) op c) op d          123    0
   *    (a op b) op (c op d)       132/312   1
   *    (a op (b op c)) op d          213    2
   *    a op ((b op c) op d)          231    3
   *    a op (b op (c op d))          321    4
   */
  public int getAssociativity() {
    return this.associativity;
  }
  
  /**
   * Sets type of associativity.
   */
  public void setAssociativity(int associativity) {
    this.associativity = associativity;
    trigger();
  }
  
  /**
   * conveniance functions to simplify event routing
   */
  public void setFirstArg(double val) {
    setInputValues(0, val);
  }
  public double getFirstArg() {
    return getInputValues(0);
  }
  public void setSecondArg(double val) {
    setInputValues(1, val);
  }
  public double getSecondArg() {
    return getInputValues(1);
  }
  public void setThirdArg(double val) {
    setInputValues(2, val);
  }
  public double getThirdArg() {
    return getInputValues(2);
  }
  public void setFourthArg(double val) {
    setInputValues(3, val);
  }
  public double getFourthArg() {
    return getInputValues(3);
  }
  public double getResult() {
    return getOutputValues(0);
  }
}
