package physbeans.func;

import java.io.*;
import physbeans.math.*;
import physbeans.model.*;

/**
 * Computes fourier coefficients of periodic functions of type
 * PeriodicFunctionWithParameter.
 * Returns cosine and sine coefficients as well as spectral amplitudes and
 * getPhase shifts.
 * Informs listeners via a VectorEvent.
 */
public class FFTFunction extends GenericVectorFunction implements Serializable {
  // uses internal function instead of vector of function values. This
  // allows to compute the FFT with a given fixed number of points (1024).
  // Otherwise the user of the bean has to care for a suitable input vector.
  
  /** Function to compute FFT from. */
  protected PeriodicFunctionWithParameter function;
  
  /** Vector of complex FFT coefficients  */
  CVector X;
  
  /** Creates a new instance of FFTFunction */
  public FFTFunction() {
    super(0, 4, 20);
    X = new CVector(1024);
    function = new SquareFunction();
  }
  
  /**
   * computes new values for output vector.
   */
  protected void compute() {
    // compute vector of function values
    double T = 2*Math.PI;    // chosen arbitrarily
    int N = getNPoints();
    DVector t = new DVector(N);
    for (int i = 0; i < N; i++) {
      t.set(i, i*T/N);
    }
    function.setPeriod(T);
    function.setInputVector(t);
    DVector x = function.getOutputVector();
    
    // compute FFT
    X = FFT.transform(new CVector(x));
    
    // compute all four types of coefficients
    int maxHarm = getNHarmonics() - 1;
    outputValues[0] = FFT.getCosineCoeffs(X).getDVector(0, maxHarm);
    outputValues[1] = FFT.getSineCoeffs(X).getDVector(0, maxHarm);
    outputValues[2] = FFT.getSpectralAmplitudes(X).getDVector(0, maxHarm);
    outputValues[3] = FFT.getSpectralPhases(X).getDVector(0, maxHarm);
  }
  
  /**
   * Gets function for fourier transformation.
   */
  public PeriodicFunctionWithParameter getFunction() {
    return this.function;
  }
  
  /**
   * Sets function for fourier transformation.
   */
  public void setFunction(PeriodicFunctionWithParameter function) {
    this.function = function;
    
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * Gets number of fourier coefficients.
   */
  public int getNHarmonics() {
    return outputValues[0].getDimension();
  }
  
  /**
   * Sets number of fourier coefficients.
   */
  public void setNHarmonics(int NHarmonics) {
    setAllOutputLengths(NHarmonics);
    
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * Gets number of points for fft.
   */
  public int getNPoints() {
    return X.getDimension();
  }
  
  /**
   * Sets number of points for fft.
   * It is set automatically to the next power of 2.
   */
  public void setNPoints(int NPoints) {
    int log2N = (int)( Math.log((double)NPoints)/Math.log(2) + 0.5 );
    int n = (int)Math.round(Math.pow(2, log2N));
    X = new CVector(n);
    
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * Gets the parameter value of the function.
   */
  public double getParameter() {
    return function.getParameter();
  }
  
  /**
   * Sets the parameter of the function.
   */
  public void setParameter(double p) {
    function.setParameter(p);
    
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * Gets coefficients of cosine series.
   */
  public DVector getCosineCoefficients() {
    return outputValues[0];
  }
  
  /**
   * Gets coefficients of sine series.
   */
  public DVector getSineCoefficients() {
    return outputValues[1];
  }
  
  /**
   * Gets spectral amplitudes of fourier series.
   */
  public DVector getAmplitudes() {
    return outputValues[2];
  }
  
  /**
   * Gets getPhase shifts of fourier series.
   */
  public DVector getPhaseShifts() {
    return outputValues[3];
  }
}
