package physbeans.inout;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import physbeans.model.*;
import physbeans.event.*;
import physbeans.math.*;

/**
 * Input panel to edit a vector of 2d points.<br>
 * Provides two different ways to change the points:
 *    <ul><li>add points incrementally / clear the whole list</li>
 *        <li>change a point given by its index</li></ul>
 * Two text sliders are affiliated with the PointEditor to provide the 
 * coordinates of the points.
 * <P>
 * The PointEditor provides the points as a Point2dVector and sends a trigger 
 * event for each change of the points.
 */
public class PointEditor extends PhysicsPanel implements Serializable  {
  
  protected JLabel titleLabel;
  protected JButton changeButton;
  protected JButton addButton;
  protected JButton clearButton;
  protected JCheckBox modifyCheckbox;
  
  protected TextSlider textSlider1;
  protected TextSlider textSlider2;
  protected JSpinner spinner;
  protected SpinnerNumberModel spinnerModel;
  protected Point2dVector points;
  
  /**
   * creates PointEditor with an empty set of points and default values
   */
  public PointEditor() {
    super();
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    points = new Point2dVector();
    
    int buttonWidth = 90;
    int buttonHeight = 30;
    int spinnerWidth = 90;
    int spinnerHeight = 30;
    
    // add heading
    titleLabel = new JLabel("Points", JLabel.LEFT);
    titleLabel.setAlignmentX(LEFT_ALIGNMENT);
    
    // add box with add and clear button
    Box addBox = new Box(BoxLayout.X_AXIS);
    addBox.setOpaque(false);
    addBox.setAlignmentX(LEFT_ALIGNMENT);
    
    addButton = new JButton("add");
    addButton.setPreferredSize(new Dimension(buttonWidth,buttonHeight));
    addButton.setOpaque(false);
    addButton.setMargin(new Insets(2,2,2,2));
    addButton.addActionListener(new MyAddListener());
    
    clearButton = new JButton("clear");
    clearButton.setPreferredSize(new Dimension(buttonWidth,buttonHeight));
    clearButton.setOpaque(false);
    clearButton.setMargin(new Insets(2,2,2,2));
    clearButton.addActionListener(new MyClearListener());
    
    modifyCheckbox = new JCheckBox("modify:");
    modifyCheckbox.setOpaque(false);
    modifyCheckbox.addItemListener(new MyCheckBoxListener());
    
    Box changeBox = new Box(BoxLayout.X_AXIS);
    changeBox.setOpaque(false);
    changeBox.setAlignmentX(LEFT_ALIGNMENT);
    
    spinnerModel = new SpinnerNumberModel(0, 0, 0, 1);
    spinner = new JSpinner(spinnerModel);
    spinner.setPreferredSize(new Dimension(spinnerWidth,spinnerHeight));
    spinner.setMaximumSize(new Dimension(spinnerWidth,spinnerHeight));
    spinner.setEnabled(false);
    spinner.addChangeListener(new MySpinnerListener());
    
    changeButton = new JButton("change");
    changeButton.setPreferredSize(new Dimension(buttonWidth,buttonHeight));
    changeButton.setOpaque(false);
    changeButton.setMargin(new Insets(2,2,2,2));
    changeButton.setEnabled(false);
    changeButton.addActionListener(new MyChangeListener());
    
    setAlignmentX(CENTER_ALIGNMENT);
    setBordered(true);
    
    // build GUI
    add(titleLabel);
    add(Box.createVerticalGlue());
    addBox.add(addButton);
    addBox.add(Box.createHorizontalGlue());
    addBox.add(clearButton);
    addBox.add(Box.createHorizontalGlue());
    add(addBox);
    add(Box.createVerticalGlue());
    add(new JSeparator());
    add(modifyCheckbox);
    changeBox.add(spinner);
    changeBox.add(Box.createHorizontalGlue());
    changeBox.add(changeButton);
    changeBox.add(Box.createHorizontalGlue());
    add(changeBox);
    
    trigSup = new TriggerSupport();
  }
  
  /**
   * gets underlying model defining the points to paint
   */
  public Point2dVector getPoints() {
    return points;
  }
  
  /**
   * sets underlying model defining the points to paint
   */
  public void setModel(Point2dVector model) {
    points = model;
  }
  
  /**
   * gets textSlider1 used for x coordinate
   */
  public TextSlider getTextSlider1() {
    return this.textSlider1;
  }
  
  /**
   * sets textSlider1 used for x coordinate
   */
  public void setTextSlider1(TextSlider textSlider1) {
    this.textSlider1 = textSlider1;
  }
  
  /**
   * gets textSlider2 used for x coordinate
   */
  public TextSlider getTextSlider2() {
    return this.textSlider2;
  }
  
  /**
   * sets textSlider2 used for x coordinate
   */
  public void setTextSlider2(TextSlider textSlider2) {
    this.textSlider2 = textSlider2;
  }
  
  /**
   * Getter for property title.
   * @return Value of property title.
   */
  public String getTitle() {
    return titleLabel.getText();
  }
  
  /**
   * Setter for property title.
   * @param title New value of property title.
   */
  public void setTitle(String title) {
    titleLabel.setText(title);
  }
  
  /**
   * Getter for property clearLabel.
   * @return Value of property clearLabel.
   */
  public String getClearLabel() {
    return clearButton.getText();
  }
  
  /**
   * Setter for property clearLabel.
   * @param clearLabel New value of property clearLabel.
   */
  public void setClearLabel(String clearLabel) {
    clearButton.setText(clearLabel);
  }
  
  /**
   * Getter for property addButtonLabel.
   * @return Value of property addButtonLabel.
   */
  public String getAddButtonLabel() {
    return addButton.getText();
  }
  
  /**
   * Setter for property addButtonLabel.
   * @param addButtonLabel New value of property addButtonLabel.
   */
  public void setAddButtonLabel(String addButtonLabel) {
    addButton.setText(addButtonLabel);
  }
  
  /**
   * Getter for property changeButtonLabel.
   * @return Value of property changeButtonLabel.
   */
  public String getChangeButtonLabel() {
    return changeButton.getText();
  }
  
  /**
   * Setter for property changeButtonLabel.
   * @param changeButtonLabel New value of property changeButtonLabel.
   */
  public void setChangeButtonLabel(String changeButtonLabel) {
    changeButton.setText(changeButtonLabel);
  }
  
  /**
   * Getter for property modifyLabel.
   * @return Value of property modifyLabel.
   */
  public String getModifyLabel() {
    return modifyCheckbox.getText();
  }
  
  /**
   * Setter for property modifyLabel.
   * @param modifyLabel New value of property modifyLabel.
   */
  public void setModifyLabel(String modifyLabel) {
    modifyCheckbox.setText(modifyLabel);
  }
  
  /**
   * action for add button
   */
  public void doAdd() {
    points.addPoint(textSlider1.getValue(), textSlider2.getValue());
    spinnerModel.setMaximum(new Integer(points.size() - 1));
    points.setActivePoint(-1);
    trigger();
  }
  
  /**
   * action for clear button
   */
  public void doClear() {
    points.clear();
    spinnerModel.setMaximum(new Integer(0));
    spinnerModel.setValue(new Integer(0));
    points.setActivePoint(-1);
    trigger();
  }
  
  public void doLayout() {
    // AFTER definition of text or setting of value
    // best at layout time so that a font is already defined from the parent
    super.doLayout();
    titleLabel.setFont(null);
    addButton.setFont(null);
    clearButton.setFont(null);
    modifyCheckbox.setFont(null);
    changeButton.setFont(null);
  }
  
  /** listener to add button */
  protected class MyAddListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      doAdd();
    }
  }
  
  /** listener to clear button */
  protected class MyClearListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      doClear();
    }
  }
  
  /** listener to modify checkbox */
  protected class MyCheckBoxListener implements ItemListener {
    public void itemStateChanged(ItemEvent itemEvent) {
      boolean state = modifyCheckbox.isSelected();
      spinner.setEnabled(state);
      changeButton.setEnabled(state);
      
      if (state) {
        int n = spinnerModel.getNumber().intValue();
        points.setActivePoint(n);
      } else {
        points.setActivePoint(-1);
      }
      trigger();
    }
  }
  
  /** listener to modify checkbox */
  protected class MySpinnerListener implements ChangeListener {
    public void stateChanged(ChangeEvent changeEvent) {
      int n = spinnerModel.getNumber().intValue();
      points.setActivePoint(n);
      trigger();
    }
  }
  
  /** listener to change button */
  protected class MyChangeListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      int n = spinnerModel.getNumber().intValue();
      DVector v = new DVector(textSlider1.getValue(), textSlider2.getValue());
      points.setPoint(n, v);
      trigger();
    }
  }
  
  // the usual support for Trigger events
  protected transient TriggerSupport trigSup;
  
  /** add a TriggerListener */
  public synchronized void addTriggerListener(TriggerListener l) {
    trigSup.addTriggerListener(l);
  }
  
  /** remove a TriggerListener */
  public synchronized void removeTriggerListener(TriggerListener l) {
    trigSup.removeTriggerListener(l);
  }
  
  /** fire a Trigger event */
  public void fireTriggerEvent(TriggerEvent e) {
    trigSup.fireTriggerEvent(e);
  }
  
  /**
   * Informs listeners of global changes.<br>
   * Useful, if a whole range of values has to be recomputed,
   *    f.i. after a parameter change.
   */
  public void trigger() {
    fireTriggerEvent(new TriggerEvent(this));
  }
}
