package physbeans.inout;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import physbeans.model.*;
import physbeans.math.*;
import physbeans.event.*;

/**
 * Generic screen class providing basic infrastructure for all kinds of screens:
 * <ul>
 *   <li>transformation between screen and world coordinates, including
 *       simple methods to change the scale,</li>
 *   <li>listener to mouse clicks, sending corresponding world coordinates
 *       as VectorEvents</li>
 *   <li>option to display a crosshair cursor.</li>
 * </ul>
 */
public class Screen extends PhysicsPanel
        implements Serializable, SizeAdjustable {

  protected ScreenWorldTrafo trafo;
  protected DVector clickedPosition;
  protected boolean crossCursor;

  /**
   * Creates Screen with default world area.
   */
  public Screen() {
    super();

    trafo = new ScreenWorldTrafo(-1.0, -1.0, 1.0, 1.0);
    clickedPosition = new DVector(2);
    crossCursor = false;

    addMouseListener(new ClickListener());
  }

  /**
   * Sets parameters when the layout (basically the size) is fixed.
   */
  public void doLayout() {
    super.doLayout();

    // set transformation parameters
    trafo.setMaxScreen(new Point(getSize().width, getSize().height));
  }

  /**
   * Just paints the background.
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
  }

  /**
   * Sets default size for GUI builder.
   */
  public Dimension getPreferredSize() {
    return new Dimension(100, 100);
  }

  /**
   * Gets maximal world x coordinate.
   */
  public double getMaxX() {
    return trafo.getMaxX();
  }

  /**
   * Sets maximal world x coordinate.
   */
  public void setMaxX(double xMax) {
    trafo.setMaxX(xMax);
    repaint();
  }

  /**
   * Gets minimal world x coordinate.
   */
  public double getMinX() {
    return trafo.getMinX();
  }

  /**
   * Sets minimal world x coordinate.
   */
  public void setMinX(double xMin) {
    trafo.setMinX(xMin);
    repaint();
  }

  /**
   * Gets maximal world y coordinate.
   */
  public double getMaxY() {
    return trafo.getMaxWorld().get(1);
  }

  /**
   * Sets maximal world y coordinate.
   */
  public void setMaxY(double yMax) {
    trafo.setMaxY(yMax);
    repaint();
  }

  /**
   * Gets minimal world y coordinate.
   */
  public double getMinY() {
    return trafo.getMinY();
  }

  /**
   * Sets minimal world y coordinate.
   */
  public void setMinY(double yMin) {
    trafo.setMinY(yMin);
    repaint();
  }

  /**
   * Gets maximal screen s coordinate.
   */
  public double getMaxS() {
    return trafo.getMaxS();
  }

  /**
   * Gets maximal screen t coordinate.
   */
  public double getMaxT() {
    return trafo.getMaxT();
  }

  /**
   * Returns true, if x and y scale are equal.
   */
  public boolean isEqualScales() {
    return trafo.isConstrained();
  }

  /**
   * Forces x and y scales to be equal or releases this constraint.
   */
  public void setEqualScales(boolean equalScales) {
    trafo.setConstrained(equalScales);
    repaint();
  }

  /**
   * Gets xyExchanged flag.
   * If true, x is vertical, y horizontal.
   */
  public boolean isXyExchanged() {
    return trafo.isXyExchanged();
  }

  /**
   * Sets xyExchanged flag.
   */
  public void setXyExchanged(boolean b) {
    trafo.setXyExchanged(b);
  }

  /**
   * Gets cursor flag: true for crosshair, false for default cursor.
   */
  public boolean isCrossCursor() {
    return this.crossCursor;
  }

  /**
   * Sets cursor flag: true for crosshair, false for default cursor.
   */
  public void setCrossCursor(boolean crossCursor) {
    this.crossCursor = crossCursor;
    if (crossCursor) {
      setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
    } else {
      setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
    repaint();
  }

  /**
   * Gets position of last mouse click in world coordinates.
   */
  public DVector getClickedPosition() {
    return this.clickedPosition;
  }

  /**
   * Changes the x scale by a given factor.
   * Only the first component of the parameter vector is used.
   * Conveniance function to simplify connection with ScreenControls.
   */
  public void changeXScale(DVector v) {
    changeXScale(v.get(0));
  }

  /**
   * Changes the y scale by a given factor.
   * Only the first component of the parameter vector is used.
   * Conveniance function to simplify connection with ScreenControls.
   */
  public void changeYScale(DVector v) {
    changeYScale(v.get(0));
  }

  /**
   * Changes the x scale by a given factor.
   */
  public void changeXScale(double d) {
    double maxX = getMaxX();
    double minX = getMinX();
    maxX *= d;
    minX *= d;
    setMaxX(maxX);
    setMinX(minX);
  }

  /**
   * Changes the y scale by a given factor.
   */
  public void changeYScale(double d) {
    double maxY = getMaxY();
    double minY = getMinY();
    maxY *= d;
    minY *= d;
    setMaxY(maxY);
    setMinY(minY);
  }

  /**
   * Changes the x and y scales by a given factor.
   * If the vector has only one element, the same factor is applied to x
   * and y direction. Otherwise the first two elements of the
   * parameter vector are used. <br>
   * Conveniance function to simplify connection with ScreenControls.
   */
  public void changeXYScale(DVector v) {
    double vx = v.get(0);
    double vy;
    if (v.getDimension() == 1) {
      vy = v.get(0);
    } else {
      vy = v.get(1);
    }

    changeXScale(vx);
    changeYScale(vy);
  }

  /**
   * computes world coordinates from given point.
   * This method is isolated so that children can provide a replacement
   * (cf. OsciScreen).
   */
  protected DVector pointToWorld(Point p) {
    return trafo.transform(p);
  }

  /**
   * Simulates a mouse click for testing purposes.
   */
  public void doClick(Point p) {
    clickedPosition = pointToWorld(p);
    fireVectorEvent(new VectorEvent(this, clickedPosition));
  }

  /**
   * listens to mouse clicks and sends corresponding world coordinates
   */
  protected class ClickListener extends MouseAdapter {

    public void mouseClicked(MouseEvent me) {
      // create vector event with the world coordinates
      clickedPosition = pointToWorld(me.getPoint());
      fireVectorEvent(new VectorEvent(this, clickedPosition));
    }
  }
  // the usual support for VectorChange events
  protected transient VectorSupport vecSup = new VectorSupport();

  /** register vector listener
   */
  public synchronized void addVectorListener(VectorListener listener) {
    vecSup.addVectorListener(listener);
  }

  /** de-register vector listener
   */
  public synchronized void removeVectorListener(VectorListener listener) {
    vecSup.removeVectorListener(listener);
  }

  /** fires vector event
   */
  public void fireVectorEvent(VectorEvent e) {
    vecSup.fireVectorEvent(e);
  }
}