package physbeans.math;

import static java.lang.Math.*; 

/**
 * Solves the generalized eigenvalue problem for two positive real matrices 
 * C, M, i.e, finds a matrix &Phi; and a diagonal matrix D so that
 * <pre>     C &Phi; = M &Phi; D </pre>
 * Eigenvalues are ordered in increasing size.
 * <p>Quick and dirty implementation: Simply reduces the problem to a standard
 * Eigenvalue problem by transforming with M<sup>(-1/2)</sup> without any checks
 * for positivity or numerical stability.
 */
public class GeneralizedEigenvalueDecomposition {
  
  protected Matrix Phi;
  protected DVector d;
  
  /** 
   * Creates a GeneralizedEigenvalueDecomposition for the matrices C and M.
   */
  public GeneralizedEigenvalueDecomposition(Matrix C, Matrix M) {
    Matrix T = invSqrt(M);
    Matrix B = T.times(C.times(T));
    
    EigenvalueDecomposition eigB = B.eig();
    Matrix V = eigB.getV();
    Phi = T.times(V);
    d = new DVector(eigB.getRealEigenvalues());
    
    sortEigenvalues();
  }
  
  /**
   * Returns the generalized eigenvector matrix &Phi;
   */
  public Matrix getPhi() {
    return Phi;
  }
  
  /**
   * Returns the generalized eigenvalues.
   */
  public DVector getEigenvalues() {
    return d;
  }
  
  /**
   * Returns diagonal matrix with generalized eigenvalues.
   */
  public Matrix getD() {
    return d.diag();
  }
  
  /**
   * Returns the inverse of the square root of matrix A.
   * Assumes A is positive and invertible without checking.
   */
  protected Matrix invSqrt(Matrix A) {
    EigenvalueDecomposition eigA = A.eig();
    Matrix VA = eigA.getV();
    double[] dA = eigA.getRealEigenvalues();
    
    for (int i = 0; i < dA.length; i++) {
      dA[i] = 1.0/sqrt(dA[i]);
    }
    Matrix DA = new DVector(dA).diag();
    
    return VA.times(DA.times(VA.transpose()));
  }
  
  /**
   * Sorts eigenvalues in ascending order.
   * Sorts eigenvectors accordingly.
   */
  protected void sortEigenvalues() {
    int[] index = d.sortWithIndex();
    int dim = index.length;
    
    Matrix PhiNew = new Matrix(dim, dim);
    for (int i = 0; i < dim; i++) {
      DVector col = Phi.getColumnVector(i);
      PhiNew.setColumnVector(index[i], col);
    }
    Phi = PhiNew;
  }
}
