package physbeans.model;

import java.io.*;

/**
 * model consisting of three doubles minVal <= value <= maxVal<br>
 * changes to model parameters leading to an inconsistent state are not
 *  simply rejected, but lead to a rearrangement of all three values.
 *  This should allow for simpler changes of the internal model.
 *
 */
public class BoundedRealModel implements Serializable {
  
  // model: minVal <= value <= maxVal
  // for a logarithmic slider variables contain the logarithms!
  protected double minVal;
  protected double maxVal;
  protected double value;
  
  // smallest difference of minval and maxval in case of inconsistent state
  static protected double diff = 0.01;
  // smallest value of minVal in case of inconsistent logarithmic slider
  static protected double eps = 1.0e-6;
  
  /** Creates a new instance of BoundedRealModel */
  public BoundedRealModel() {
    this(0.0, 0.5, 1.0);
  }
  
  /** copy constructor, useful for PropertyEditor */
  public BoundedRealModel(BoundedRealModel model) {
    minVal = model.getMinimum();
    value = model.getValue();
    maxVal = model.getMaximum();
    checkModel();
  }

  public BoundedRealModel(double min, double val, double max) {
    minVal = min;
    value = val;
    maxVal = max;
    checkModel();
  }
  
  /** returns maximal value */
  public double getMaximum() {
    return maxVal;
  }
  
  /** returns maximal value */
  public double getMinimum() {
    return minVal;
  }
  
  /** returns maximal value */
  public double getValue() {
    return value;
  }
  
  /** sets upper bound to max. <br>
   *  repairs upper bound and value if necessary
   */
  public void setMaximum(double max) {
    maxVal = max;
    checkModel();
  }

  /** sets lower bound to min. <br>
   *  repairs upper bound and value if necessary
   */
  public void setMinimum(double min) {
    minVal = min;
    checkModel();
  }

  /** sets value to val. <br>
   *  repairs value if out of bounds
   */
  public void setValue(double val) {
    value = val;
    checkModel();
  }
  
  /** sets bounds and value in one step. <br>
   *  repairs upper bound and value if necessary
   */
  public void setModel(double min, double val, double max) {
    minVal = min;
    value = val;
    maxVal = max;
    checkModel();    
  }
  
  /** rearrange internal model to retain consistent state */
  protected void checkModel() {
    // guarantees that
    //    minval <= value <= maxval
    //    minval != maxval
    
    // check maxval
    if (maxVal <= minVal) {
      maxVal = minVal + diff;
    }
    
    // check value
    value = Math.max(Math.min(value, maxVal), minVal);
  }
}
