package physbeans.model;

import physbeans.math.*;
import physbeans.phys.*;

/**
 * Container of a number of data buffers of equal size.
 * Arriving data is stored incrementally. <br>
 * When the buffer is full, it is used cyclically, i.e. new data is stored at
 * the beginning and overwrites the oldest data elements.
 */
public class DataBuffer extends PhysModel {
  
  protected int nBuf;
  protected int size;
  
  protected DVector[] data;
  protected int pos;            // index for next data
  
  /** Creates a new instance of DataBuffer */
  public DataBuffer() {
    super();
    nBuf = 2;
    size = 100;
    pos = 0;
    createBuffer();
  }
  
  /**
   * Stores the data in d at the next position.
   */
  public void storeData(DVector d) {
    if (d.getDimension() != nBuf) {
      return;
    }
    
    if (pos == size) {
      pos = 0;
    }
    
    for (int i = 0; i < nBuf; i++) {
      data[i].set(pos, d.get(i));
    }
    
    pos++;
    trigger();
  }
  
  /**
   * Gets all data.
   */
  public DVector[] getData() {
    return data;
  }
  
  /**
   * Gets data set n.
   */
  public DVector getData(int n) {
    return data[n];
  }
  
  /**
   * Gets number of data elements in the buffer.<br>
   * Possible values are 0 .. size.
   */
  public int getNData() {
    return pos;
  }
  
  
  /**
   * Restarts the buffer.
   */
  public void restart() {
    pos = 0;
    trigger();
  }
  
  /**
   * Gets the number of data buffers.
   */
  public int getNBuffer() {
    return nBuf;
  }
  
  /**
   * Sets the number of data buffers.
   */
  public void setNBuffer(int n) {
    nBuf = n;
    createBuffer();
    trigger();
  }
  
  /**
   * Gets the size of the data buffers.
   */
  public int getSize() {
    return size;
  }
  
  /**
   * Sets the size of the data buffers.
   */
  public void setSize(int n) {
    size = n;
    createBuffer();
    trigger();
  }
  
  /**
   * Gets the latest data elements.
   */
  public DVector getLastData() {
    DVector v;
    
    if (pos != 0) {
      v = new DVector(nBuf);
      for (int i = 0; i < nBuf; i++) {
        v.set(i, data[i].get(pos - 1));
      }
    } else {
      v = new DVector(0);
    }
    return v;
  }
  
  /**
   * Creates new buffers of given size.
   */
  protected void createBuffer() {
    data = new DVector[nBuf];
    for (int i = 0; i < nBuf; i++) {
      data[i] = new DVector(size);
    }
  }
  
}
