package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;
import physbeans.model.*;
import java.io.*;

/**
 * A physical bean describing a driven harmonic spring with external force
 * and gravity according to the following ODE:<br>
 *    m x'' + b x' + c x = -m g + F(t)
 * <p>
 * It has the following parameters
 *  <ul>
 * <li>mass m</li>
 * <li>stiffness c</li>
 * <li>friction coefficient b</li>
 * <li>gravity constant g</li>
 * </ul>
 * <p>An arbitrary external force F can be added via a Generic1dFunction which is
 * associated with the DrivenSpring. The force function may be called several
 * times for one time step due to step adaptation. These calls are done in the
 * stateless mode, so that they don't produce lots of events for external 
 * listeners.</p>
 * <p>DrivenSpring stores initial values and can be restarted. If an external
 * force function is used, it is automatically restarted then (sending its new
 * values to potential listeners!).</p>
 */
public class DrivenSpring extends SolveODEFunction implements Serializable {
  
  protected double m;
  protected double c;
  protected double b;
  protected double g;
  
  /** function that computes time depending values of the external torque */
  protected Real1DFunction externalForceFunction;
  
  /** Creates a new instance of DrivenSpring */
  public DrivenSpring() {
    super();
    
    m = 1.0;
    c = 1.0;
    b = 0.0;
    g = 0.0;
    
    setOde(new DrivenSpringODE());
    externalForceFunction = null; // by default, torque = 0.0
    restart();
  }
  
  /**
   * Gets mass m attached to the spring.
   */
  public double getM() {
    return this.m;
  }
  
  /**
   * Sets mass m attached to the spring.
   */
  public void setM(double m) {
    this.m = m;
    restart();
  }
  
  /**
   * Gets stiffness c of spring.
   */
  public double getC() {
    return this.c;
  }
  
  /**
   * Sets stiffness c of spring.
   */
  public void setC(double c) {
    this.c = c;
    restart();
  }
  
  /**
   * Gets viscous damping coefficient b.
   */
  public double getB() {
    return this.b;
  }
  
  /**
   * Sets viscous damping coefficient b.
   */
  public void setB(double b) {
    this.b = b;
    restart();
  }
  
  /**
   * Gets free fall acceleration g.
   */
  public double getG() {
    return this.g;
  }
  
  /**
   * Sets free fall acceleration g.
   */
  public void setG(double g) {
    this.g = g;
    restart();
  }
  
  /**
   * Gets initial position x0.
   */
  public double getX0() {
    return getInitialValue(0);
  }
  
  /**
   * Sets initial position x0.
   */
  public void setX0(double x0) {
    setInitialValue(0, x0);
  }
  
  /**
   * Gets initial velocity v0.
   */
  public double getV0() {
    return getInitialValue(1);
  }
  
  /**
   * Sets initial velocity v0.
   */
  public void setV0(double v0) {
    setInitialValue(1, v0);
  }
  
  /**
   * Gets function that computes time depending values of the external torque
   */
  public Real1DFunction getExternalForceFunction() {
    return this.externalForceFunction;
  }
  
  /**
   * Sets function that computes time depending values of the external torque
   */
  public void setExternalForceFunction(Real1DFunction externalForceFunction) {
    this.externalForceFunction = externalForceFunction;
  }
  
  
  class DrivenSpringODE extends ODE {
    public DrivenSpringODE() {
      super(new DVector(1.0, 0.0), 0.0);
    }
    
    public DVector f(DVector x, double t) {
      DVector result = new DVector(2);
      
      // get external force as value of the externalForceFunction
      double force;
      if (externalForceFunction != null) {
        force = externalForceFunction.evaluate(t);
      } else {
        force = 0.0;
      }
      
      result.set(0, x.get(1));   // x(0)' = x(1) = v
      result.set(1, force/m - g -(c/m)*x.get(0) - (b/m)*x.get(1));
      return result;
    }
    
  }
}
