package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;
import physbeans.model.*;

import java.io.*;

/**
 * A physical bean describing a Duffing Oscillator with external force
 * according to the following ODE:<br>
 *    x'' + &lambda;x' + &mu;x + &nu;x<sup>3</sup> = F(t)
 * <p>An arbitrary external force F can be added via a OneParameterFunction,
 * which is associated with the DuffingOscillator through the property
 * ExternalForceFunction.
 * <br>
 * The force function may be called several times for one time step due to
 * step adaptation. These calls are done in the stateless mode, so that they
 * don't produce lots of events for external listeners.</p>
 * <p>DuffingOscillator stores initial values and can be restarted. If an external
 * force function is used, it is automatically restarted then (sending its new
 * values to potential listeners!).</p>
 */
public class DuffingOscillator extends SolveODEFunction implements Serializable {
  
  protected double mu;
  protected double nu;
  protected double lambda;
  protected Real1DFunction externalForceFunction;
  
  /** Creates a new instance of DuffingOscillator */
  public DuffingOscillator() {
    super();
    
    lambda = 1.0;
    mu = 1.0;
    nu = 1.0;
    
    setOde(new DuffingODE());
    externalForceFunction = null; // by default, force = 0.0
    restart();
  }
  
  /**
   * Gets parameter &mu; of the Duffing ODE.
   */
  public double getMu() {
    return this.mu;
  }
  
  /**
   * Sets parameter &mu; of the Duffing ODE.
   */
  public void setMu(double mu) {
    this.mu = mu;
    restart();
  }
  
  /**
   * Gets parameter &nu; of the Duffing ODE.
   */
  public double getNu() {
    return this.nu;
  }
  
  /**
   * Sets parameter &nu; of the Duffing ODE.
   */
  public void setNu(double nu) {
    this.nu = nu;
    restart();
  }
  
  /**
   * Gets parameter &lambda; of the Duffing ODE.
   */
  public double getLambda() {
    return this.lambda;
  }
  
  /**
   * Sets parameter &lambda; of the Duffing ODE.
   */
  public void setLambda(double lambda) {
    this.lambda = lambda;
    restart();
  }
  
  /**
   * Gets initial displacement x0.
   */
  public double getX0() {
    return getInitialValue(0);
  }
  
  /**
   * Sets initial displacement x0.
   */
  public void setX0(double x0) {
    setInitialValue(0, x0);
  }
  
  /**
   * Gets initial velocity v0.
   */
  public double getV0() {
    return getInitialValue(1);
  }
  
  /**
   * Sets initial velocity v0.
   */
  public void setV0(double v0) {
    setInitialValue(1, v0);
  }
  
  /**
   * Gets the function that computes time depending values of the external force.
   */
  public Real1DFunction getExternalForceFunction() {
    return this.externalForceFunction;
  }
  
  /**
   * Sets the function that computes time depending values of the external force.
   */
  public void setExternalForceFunction(Real1DFunction externalForceFunction) {
    this.externalForceFunction = externalForceFunction;
  }
  
  /**
   * Gets the current value of the displacement.<p>
   * Conveniance function to simplify routing
   */
  public double getX() {
    return getOutputValues(0);
  }
  
  /**
   * Gets the current value of the velocity.<p>
   * Conveniance function to simplify routing
   */
  public double getV() {
    return getOutputValues(1);
  }
  
  class DuffingODE extends ODE {
    public DuffingODE() {
      super(new DVector(1.0, 0.0), 0.0);
    }
    
    public DVector f(DVector x, double t) {
      DVector result = new DVector(2);
      
      // get external force
      double force;
      if (externalForceFunction != null) {
        force = externalForceFunction.evaluate(t);
      } else {
        force = 0.0;
      }
      
      double xa = x.get(0);
      double xb = x.get(1);
      result.set(0, xb);
      result.set(1, -lambda*xb - mu*xa - nu*xa*xa*xa + force);
      return result;
    }
    
  }
}
