package physbeans.phys;

import physbeans.math.*;
import static java.lang.Math.*;

/**
 * Creates a gaussian wave packet as initial value on a 2d complex grid.
 * Explicitely it is given by
 *   psi(x) = N * exp(-0.5*((x-x0)/s)^2)*exp(i*(ME/HBAR)*v0*x)
 * for 2d-vectors x, x0, v0, scalar s and normalisation constant N.
 */
public class GaussianGridInitializer extends ComplexGridInitializer {

  protected DVector x0;
  protected DVector v0;
  protected double s;

  public GaussianGridInitializer() {
    super();
    x0 = new DVector(0.0, 0.0);
    v0 = new DVector(0.0, 0.0);
    s = 1.0;
  }

  /**
   * Sets initial values for the ScalarField f.
   */
  public void setGrid() {
    if (field == null) {
      return;
    }

    double[][] gridR = field.getGridReal();
    double[][] gridI = field.getGridImag();

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    int bwX = field.getBoundaryWidthX();
    int bwY = field.getBoundaryWidthY();

    clearBoundaries();

    for (int i = bwX; i < nx - bwX; i++) {
      for (int j = bwY; j < ny - bwY; j++) {
        DVector x = field.getWorldFromIndex(i, j);
        double phi = (Constants.ME / Constants.HBAR) * v0.times(x);
        DVector t1 = x.minus(x0).times(1.0 / s);
        double r = exp(-0.5 * t1.times(t1));

        gridR[j][i] = r * cos(phi);
        gridI[j][i] = r * sin(phi);
      }
    }

    normalizeField();
  }

  /**
   * Sets initial position x0 of the wave packet.
   */
  public void setX0(DVector x) {
    x0 = x;
    updateField();
  }

  /**
   * Gets initial position x0 of the wave packet.
   */
  public DVector getX0() {
    return x0;
  }

  /**
   * Sets initial velocity v0 of the wave packet.
   */
  public void setV0(DVector v) {
    v0 = v;
    updateField();
  }

  /**
   * Gets initial velocity v0 of the wave packet.
   */
  public DVector getV0() {
    return v0;
  }

  /**
   * Sets width (standard deviation) of the wave packet.
   */
  public void setWidth(double sNew) {
    s = sNew;
    updateField();
  }

  /**
   * Gets width (standard deviation) of the wave packet.
   */
  public double getWidth() {
    return s;
  }
}
