package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;
import java.io.*;

/**
 * A physical bean describing a mass point in one dimension that is subject to
 * an arbitrary force F(x, v, t) according to the following ODE:<br>
 *    m x'' = F(x, v, t)
 * <p>
 * It has the following parameters
 * <ul>
 * <li>mass m</li>
 * <li>force function F defined by parsing a function string</li>
 * <li>a list of parameter names that can be used in the definition of F in 
 *     addition to the standard variables x, v, t, and m</li>
 * <li>values for the parameters</li>
 * </ul>
 * <p>Generic1DMassPoint stores initial values and can be restarted.</p>
 * <p>An additional flag negativeForce can be set, in which case the definition
 * string of F actually gives -F. In this case the Generic1DMassPoint actually
 * implements the equation<br>
 *   m x'' + F(x, v, t) = 0
 * </p>
 */
public class Generic1DMassPoint extends SolveODEFunction implements Serializable {
  
  protected double m;
  protected String[] parameterNames;
  protected boolean negativeForce;
  
  protected ParsedFunction parser;
  protected String[] varNames;
  
  /**
   * Creates a default instance of Generic1DMassPoint.
   * Describes a freely falling mass with g as single parameter.
   */
  public Generic1DMassPoint() {
    super();
    m = 1.0;
    negativeForce = false;
    varNames = new String[] {"x", "v", "t", "m"};
    
    parser = new ParsedFunction();
    parser.setFunction(0, "-m*g");
    parser.setParameter(new DVector(9.81));
    parser.setVariableName(new String[] {"x", "v", "t", "m", "g"});
    
    setOde(new NewtonODE());
    restart();
  }
  
  /**
   * Gets mass m attached to the spring.
   */
  public double getMass() {
    return this.m;
  }
  
  /**
   * Sets mass m attached to the spring.
   */
  public void setMass(double m) {
    this.m = m;
    restart();
  }
  
  /**
   * Gets description string of the function.
   */
  public String getFunction() {
    return parser.getFunction(0);
  }
  
  /**
   * Sets description strings of the function.
   */
  public void setFunction(String f) {
    parser.setFunction(0, f);
    restart();
  }
  
  /**
   * Gets the value of parameter i.
   */
  public double getParameter(int i) {
    return parser.getParameter(i);
  }
  
  /**
   * Gets the values of all parameters.
   */
  public DVector getParameter() {
    return parser.getParameter();
  }
  
  /**
   * Sets the value of parameter i.
   */
  public void setParameter(int i, double val) {
    parser.setParameter(i, val);
  }
  
  /**
   * Sets the values of all parameters.
   */
  public void setParameter(DVector vals) {
    parser.setParameter(vals);
  }
  
  /**
   * Gets the name of parameter i.
   */
  public String getParameterName(int i) {
    // the first variable names are fixed
    
    return parser.getVariableName(i + varNames.length);
  }
  
  /**
   * Gets the names of all parameters.
   */
  public String[] getParameterName() {
    // cut off the fixed variable names
    
    int vl = varNames.length;
    String[] vars = parser.getVariableName();
    String[] pars = new String[vars.length - vl];
    
    for (int i = 0; i < pars.length; i++) {
      pars[i] = vars[i + vl];
    }
    
    return pars;
  }
  
  /**
   * Sets the name of parameter i.
   */
  public void setParameterName(int i, String name) {
    parser.setVariableName(i + varNames.length, name);
  }
  
  /**
   * Sets the names of all parameters.
   */
  public void setParameterName(String[] names) {
    int vl = varNames.length;
    int nl = names.length;
    String[] vars = new String[vl + nl];
    
    for (int i = 0; i < vl; i++) {
      vars[i] = varNames[i];
    }
    
    for (int i = vl; i < vl+nl; i++) {
      vars[i] = names[i - vl];
    }
    
    parser.setVariableName(vars);
  }
  
  /**
   * Gets initial position x0.
   */
  public double getX0() {
    return getInitialValue(0);
  }
  
  /**
   * Sets initial position x0.
   */
  public void setX0(double x0) {
    setInitialValue(0, x0);
  }
  
  /**
   * Gets initial velocity v0.
   */
  public double getV0() {
    return getInitialValue(1);
  }
  
  /**
   * Sets initial velocity v0.
   */
  public void setV0(double v0) {
    setInitialValue(1, v0);
  }
  
  /**
   * Gets the value of the flag negativeForce.
   */
  public boolean isNegativeForce() {
    return negativeForce;
  }
  
  /**
   * Sets the value of the flag negativeForce.
   */
  public void setNegativeForce(boolean b) {
    negativeForce = b;
  }

  class NewtonODE extends ODE {
    public NewtonODE() {
      super(new DVector(1.0, 0.0), 0.0);
    }
    
    public DVector f(DVector x, double t) {
      DVector arg = new DVector(x.get(0), x.get(1), t, m);   // x, v, t, m
      parser.setInputVector(arg);
      double force = parser.getOutputValue();
      if (negativeForce) {
        force = -force;
      }
            
      DVector result = new DVector(2);
      result.set(0, x.get(1));        // x0' = x1 = v
      result.set(1, force/m);         // x1' = v' = F/m
      return result;
    }
    
  }
}
