package physbeans.phys;

import physbeans.model.*;
import physbeans.math.*;

/**
 * Model of a light ray that passes through a optical system, basically a
 * polyline with an additional leaving angle.
 * <P>
 * A LightRay always starts at a definitive point, given by a LightSource,
 * but usually it has no end point and leaves the system in a certain angle.
 * <P>
 * The interaction of a LightRay with an OpticalElement leads to the insertion
 * of additional points, where the ray crosses the active surfaces, and a change
 * of the end angle. In some situations an optical system can lead to a ray
 * with an infinite number of points (f.i. reflection between parallel mirrors).
 * To avoid an infinite loop, the number of points is bounded.
 * <P>
 * In case the light ray ends (f.i. if it is absorbed by an aperture or the
 * maximal number of points has been reached), this angle is set to NaN.
 * <P>
 * A LightRay has a wavelength, which can be used to describe dispersion.
 */
public class LightRay {
  
  protected static int maxPoints;
  
  protected Point2dVector points;
  protected double angle;
  protected double wavelength;
  
  /**
   * Creates a new LightRay with given start coordinates and angle.
   */
  public LightRay(double startX, double startY, double angle) {
    points = new Point2dVector();
    points.addPoint(startX, startY);
    this.angle = angle;
    
    maxPoints = 100;
    wavelength = 600e-9;
  }
  
  /**
   * Creates a new LightRay with a given start point and angle.
   */
  public LightRay(DVector p, double angle) {
    this(p.get(0), p.get(1), angle);
  }
  
  /**
   * Gets the points describing the LightRay.
   */
  public Point2dVector getPoints() {
    return points;
  }
  
  /**
   * Gets current number of points in the ray.
   */
  public int getNPoints() {
    return points.size();
  }
  
  /**
   * Adds a point at the end of the LightRay.<br>
   * If the maximal number of points has been reached already or the LightRay
   * ended before , it does nothing and returns false.
   * Otherwise it returns true.
   */
  public boolean addPoint(double x, double y) {
    if (angle == Double.NaN) {
      return false;
    }
    
    points.addPoint(x,y);
    if (points.size() == maxPoints) {
      angle = Double.NaN;
    }
    return true;
  }
  
  /**
   * Adds a point at the end of a LightRay.
   */
  public boolean addPoint(DVector p) {
    return addPoint(p.x(), p.y());
  }
  
  /**
   * Gets last point of the LightRay
   */
  public DVector getLastPoint() {
    return points.getPoint(points.size()-1);
  }
  
  /**
   * Gets the final angle of the LightRay.
   */
  public double getAngle() {
    return angle;
  }
  
  /**
   * Sets the final angle of the LightRay.<p>
   * The angle is reduced to the basic interval [-pi, pi).
   */
  public void setAngle(double phi) {
    phi -= 2*Math.PI*Math.floor(phi/(2*Math.PI));  // lies in [0, 2*pi)
    if (phi>= Math.PI) {
      phi -= 2*Math.PI;
    }
    
    angle = phi;
  }
  
  /**
   * Gets maximal number of points
   */
  public static int getMaxPoints() {
    return maxPoints;
  }
  
  /**
   * Sets maximal number of points
   */
  public static void setMaxPoints(int max) {
    maxPoints = max;
  }
  
  /**
   * Gets wavelength of the LightRay
   */
  public double getWavelength() {
    
    return this.wavelength;
  }
  
  /**
   * Sets wavelength of the LightRay
   */
  public void setWavelength(double wavelength) {
    
    this.wavelength = wavelength;
  }
  
  /**
   * Returns true if the current ray is going in positive x direction.
   */
  public boolean isRightGoing() {
    return Maths.isStrictlyPositive(Math.PI/2 - Math.abs(angle));
  }

 /**
   * Returns true if the current ray is going in negative x direction.
   */
  public boolean isLeftGoing() {
    return Maths.isStrictlyPositive(Math.abs(angle) - Math.PI/2);
  }
}
