package physbeans.phys;

import physbeans.math.*;

/**
 * Abstract base class of all elements in an optical bench that interact with
 * light rays.
 */
abstract public class OpticalElement extends PhysModel implements Comparable<OpticalElement> {
  
  protected double pos;
  
  /** Creates a new instance of OpticalElement */
  public OpticalElement() {
    super();
    pos = 0.0;
  }
  
  /**
   * Gets the position (the x coordinate of a specified point) of the
   * optical element.
   */
  public double getPosition() {
    return pos;
  }
  
  /**
   * Sets the position (the x coordinate of a specified point) of the
   * optical element.
   */
  public void setPosition(double pos) {
    this.pos = pos;
    trigger();
  }
  
  /**
   * Adds additional points and/or changes the final angle of the ray as it
   * passes through the optical element.
   */
  public void propagate(LightRay ray) {
    // too many points -> stop propagation
    if (ray.getNPoints() == ray.getMaxPoints()) {
      ray.setAngle(Double.NaN);
      return;
    }
    extend(ray);
  }
  
  /**
   * Extends a light ray through the optical element.
   */
  abstract protected void extend(LightRay ray);
  
  /**
   * Adds additional points and/or changes the final angles of the rays as they
   * pass through the  optical element.
   */
  public void propagate(LightRayBundle rays) {
    for (int i = 0; i < rays.getNRays(); i++) {
      propagate(rays.getRay(i));
    }
  }
  
  /**
   * Returns true, if a ray is departing from the (infinitely thin) OpticalElement.
   */
  protected boolean rayIsDeparting(LightRay ray) {
    double x = ray.getLastPoint().get(0);
    double al = ray.getAngle();
    
    if ( (x < pos) && (Math.abs(al) > Math.PI/2)) {
      return true;
    }
    
    if ( (x > pos) && (Math.abs(al) < Math.PI/2)) {
      return true;
    }
    
    return false;
  }
  
  /**
   * Uses the position along the optical (x) axis to compare OpticalElements.
   * Returns 1 (0, -1), if this object has a larger (equal, smaller) x position
   * than the specified object o.
   */
  public int compareTo(OpticalElement el) {
    if (Maths.isStrictlyPositive(pos - el.pos)) {
      return 1;
    } else if (Maths.isStrictlyPositive(el.pos - pos)) {
      return -1;
    } else {
      return 0;
    }
  }
}
