package physbeans.phys;

import physbeans.model.*;
import physbeans.math.*;

/**
 * Model of a spherical mirror using the paraxial approximation. <p>
 * The actual reflexion is computed at the tangential plane perpendicular to
 * the optical axis ("principal plane"). The mirror reflects at both sides and can
 * be made convex or concave to the left (or right) by choosing the sign of the
 * curvature radius.
 */
public class ParabolicMirror extends OpticalElement {
  
  protected double r;
  protected double h;
  
  /** Creates a new instance of ParabolicMirror */
  public ParabolicMirror() {
    super();
    r = -5.0;
    h = 2.0;
  }
  
  protected void extend(LightRay ray) {
    DVector pLast = ray.getLastPoint();
    double al = ray.getAngle();
    
    Parabola2D parabola = new Parabola2D(new DVector(pos, 0.0), r,
        getLeftEdge(), getRightEdge());
    Ray2D endRay = new Ray2D(pLast, al);
    DVector pHit = parabola.getSegmentRayHitPoint(endRay);
    
    if (Double.isNaN(pHit.x())) {
      return;    // miss
    }
    ray.addPoint(pHit);
    
    DVector in = new DVector(Math.cos(al), Math.sin(al));
    DVector n = parabola.getNormal(pHit);
    
    ray.setAngle(OpticalSurface.getReflectionAngle(in, n));
    propagate(ray);
  }
  
  /**
   * Gets the radius of the curvature at the vertex.
   * A positive value means the focal point of the parabola lies to the right of
   * the mirror. This is different from the usual mirror conventions, but in
   * accordance with standard lens conventions.
   */
  public double getRadius() {
    return this.r;
  }
  
  /**
   * Sets the radius of the curvature at the vertex.
   */
  public void setRadius(double r) {
    this.r = r;
    trigger();
  }
  
  /**
   * Gets the height of the mirror, measured from the optical axis.
   * The extension of the mirror (i.e. the cross section of its projection onto
   * the tangential plane) is twice the height.
   */
  public double getHeight() {
    return this.h;
  }
  
  /**
   * Sets the height of the mirror, measured from the optical axis.
   * The extension of the mirror (i.e. the cross section of its projection onto
   * the tangential plane) is twice the height.
   */
  public void setHeight(double h) {
    if (h <= Math.abs(r)) {
      this.h = h;
      trigger();
    }
  }
  
  /**
   * Returns the position of the focal point as a one element Point2dVector.
   */
  public Point2dVector getFocalPoints() {
    Point2dVector point = new Point2dVector();
    point.addPoint(pos + r/2, 0.0);
    return point;
  }
  
  /**
   * Returns the smallest x value to which the mirror extends
   */
  protected double getLeftEdge() {
    return (r > 0)? pos : pos + h*h/(2*r);
  }
  
  /**
   * Returns the largest x value to which the mirror extends
   */
  protected double getRightEdge() {
    return (r > 0)? pos + h*h/(2*r) : pos;
  }
}
