package physbeans.phys;

import physbeans.math.*;
import static java.lang.Math.*;

/**
 * Creates a free wave strictly localized in a box as initial value on a
 * 2d complex grid.
 * Explicitely it is given by
 *   psi(x) = N * Iw(x - x0) * exp(i*(ME/HBAR)*v0*(x - x0)
 * for 2d-vectors x, x0, v0 and normalisation constant N. Iw is the 
 * characteristic function of a rectangular box of width [wx, wy] centered at 0.
 */
public class PlaneWaveGridInitializer extends ComplexGridInitializer {

  protected DVector x0, v0, w;

  public PlaneWaveGridInitializer() {
    super();
    x0 = new DVector(0.0, 0.0);
    v0 = new DVector(0.0, 0.0);
    w = new DVector(1.0, 1.0);
  }

  /**
   * Sets initial values for the ScalarField f.
   */
  public void setGrid() {
    if (field == null) {
      return;
    }

    double[][] gridR = field.getGridReal();
    double[][] gridI = field.getGridImag();

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    int bwX = field.getBoundaryWidthX();
    int bwY = field.getBoundaryWidthY();

    clearBoundaries();

    for (int i = bwX; i < nx - bwX; i++) {
      for (int j = bwY; j < ny - bwY; j++) {
        DVector x = field.getWorldFromIndex(i, j);
        if ((abs(x.x() - x0.x()) <= 0.5*w.x())
                && (abs(x.y() - x0.y()) <= 0.5*w.y())) {
          double phi = (Constants.ME / Constants.HBAR) * v0.times(x.minus(x0));
          gridR[j][i] = cos(phi);
          gridI[j][i] = sin(phi);
        } else {
          gridR[j][i] = 0.0;
          gridI[j][i] = 0.0;
        }
      }
    }

    normalizeField();
  }

  /**
   * Sets initial position x0 of the box containing the wave.
   */
  public void setX0(DVector x) {
    x0 = x;
    updateField();
  }

  /**
   * Gets initial position x0 of the box containing the wave.
   */
  public DVector getX0() {
    return x0;
  }

  /**
   * Sets initial velocity v0 of the wave.
   */
  public void setV0(DVector v) {
    v0 = v;
    updateField();
  }

  /**
   * Gets initial velocity v0 of the wave.
   */
  public DVector getV0() {
    return v0;
  }

  /**
   * Sets width of the box containing the wave.
   */
  public void setWidth(DVector wNew) {
    w = wNew;
    updateField();
  }

  /**
   * Gets width of the box containing the wave.
   */
  public DVector getWidth() {
    return w;
  }
}
