package physbeans.phys;

import physbeans.math.*;
import physbeans.func.*;

/**
 * N-stage exponential decay function.
 * <P>Computes number of particles of an exponential decay chain
 * with n particle types.
 */
public class RadioactiveDecayChain  extends SolveODEFunction {
  
  protected DVector lambda;
  protected DVector n0, tHalf;  // redundant, but needed to fix NetBeans bug
  
  /**
   * Creates RadioactiveDecayChain with standard values.
   */
  public RadioactiveDecayChain() {
    super();
    n0 = new DVector(1000.0, 0.0, 0.0, 0.0);
    setOde(new RadioactiveDecayChainODE());
    
    double val = Math.log(2);
    setTHalf(new DVector(val, val, val, Double.POSITIVE_INFINITY));
  }
  
  /**
   * Gets half life times of the particle types.
   */
  public DVector getTHalf() {
    return tHalf;
  }
  
  /**
   * Sets half life times of the particle types.
   * <P>If a value is infinity the corresponding particle type is stable.
   */
  public void setTHalf(DVector tHalf) {
    this.tHalf = tHalf;
    int n = tHalf.getDimension();
    lambda = new DVector(n);
    
    for (int i = 0; i < n; i++) {
      lambda.set(i, Math.log(2)/tHalf.get(i));
    }
    restart();
  }
  
  /**
   * Sets half life time of a given particle type.
   * <p>Conveniance function to simplify routing.
   */
  public void setTHalf(int n, double tH) {
    tHalf.set(n, tH);
    lambda.set(n, Math.log(2)/tH);
  }
  
  /**
   * Gets all initial values.
   */
  public DVector getInitialValues() {
    return n0;
  }
  
  /**
   * Sets all initial values.
   */
  public void setInitialValues(DVector x0) {
    super.setInitialValues(x0);
    n0 = x0;
  }
  
  class RadioactiveDecayChainODE extends ODE {
    public RadioactiveDecayChainODE() {
      super(n0, 0.0);
    }
    
    public DVector f(DVector x, double t) {
      int n = x.getDimension();
      DVector result = new DVector(n);
      
      result.set(0, -lambda.get(0)*x.get(0));
      
      for (int i = 1; i < n; i++) {
        result.set(i, lambda.get(i-1)*x.get(i-1) - lambda.get(i)*x.get(i));
      }
      
      return result;
    }
    
  }
  
}
