package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import physbeans.math.*;

/**
 * Paints a simple bar graph representing an input vector.<br>
 * The vertical scaling and zero position is given by the underlying
 * LayeredScreen2D, the horizontal scaling by the number of bars.<br>
 * The colors of the bars are defined using one of two schemes:
 * <ul>
 * <li>equal colors for all bars, defined by barColor and lineColor,</li>
 * <li>different colors for all bars, given by a fixed set of colors.</li>
 * </ul>
 */
public class BarView extends RestartableView {
  
  protected double barWidth;
  protected Color barColor;
  protected Color lineColor;
  protected boolean equalBarColor;
  
  /** colors of the bars (used cyclically) */
  protected Color[] barColors = {Color.green, Color.magenta, Color.cyan,
  Color.pink, Color.orange, Color.red, Color.blue, Color.yellow};
  
  
  /** Creates a default BarView */
  public BarView() {
    super();
    model = new DVector(1.0, 0.5, 0.0, -0.5);
    barWidth = 0.7;
    lineColor = Color.black;
    barColor = new Color(0, 21, 127);
    equalBarColor = true;
  }
  
  public void paintLayer(Graphics2D g) {
    if (model == null) {
      return;  // do nothing
    }
    
    // compute width of one bar including gap
    int nBars = model.getDimension();
    Rectangle2D.Double bar = new Rectangle2D.Double();
    
    for (int i = 0; i < nBars; i++) {
      Color actBarColor;
      if (equalBarColor) {
        actBarColor = barColor;
      } else {
        actBarColor = barColors[i % barColors.length];
      }
      
      double value = model.get(i);
      if (value >= 0) {
        bar.setRect(i, 0, barWidth, value);
      } else {
        bar.setRect(i, value, barWidth, -value);
      }
      
      g.setColor(actBarColor);
      g.fill(bar);
      g.setColor(lineColor);
      g.draw(bar);
    }
  }
  
  /**
   * Gets width of one bar.
   */
  public double getBarWidth() {
    return this.barWidth;
  }
  
  /**
   * Sets width of one bar.
   */
  public void setBarWidth(double barWidth) {
    this.barWidth = barWidth;
    update();
  }
  
  /**
   * Gets body color of a bar.
   */
  public Color getBarColor() {
    return this.barColor;
  }
  
  /**
   * Sets body color of a bar.
   */
  public void setBarColor(Color barColor) {
    this.barColor = barColor;
    update();
  }
  
  /**
   * Gets line color of a bar.
   */
  public Color getLineColor() {
    return this.lineColor;
  }
  
  /**
   * Sets line color of a bar.
   */
  public void setLineColor(Color lineColor) {
    this.lineColor = lineColor;
    update();
  }
  
  /**
   * Gets value of flag equalBarColor.
   * If true, all bars use the same body and line color.
   * @return Value of property equalBarColor.
   */
  public boolean isEqualBarColor() {
    return this.equalBarColor;
  }
  
  /**
   * Sets value of flag equalBarColor.
   * If true, all bars use the same body and line color.
   */
  public void setEqualBarColor(boolean equalBarColor) {
    this.equalBarColor = equalBarColor;
  }
  
}

