package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * View that displays a vector of 2d points (given as a Point2dVector or two
 * DVectors) as a polyline forming a function graph.
 */
public class FunctionView extends View implements Serializable {
  // TODO: logarithmic in x or y
  
  protected DVector x, y;
  protected Color lineColor;
  
  /**
   * Creates FunctionView with an empty set of points and default values.
   */
  public FunctionView() {
    super();
    lineColor = Color.black;
  }
  
  /**
   * Paints the points as a polyline.
   */
  public void paintLayer(Graphics2D g2) {
    if ((x == null) || (y == null)) {
      return;  // do nothing
    }
    
    g2.setColor(lineColor);
    
    // draw GeneralPath (polyline)
    double[] xPoints = x.getArray();
    double[] yPoints = y.getArray();
    
    GeneralPath polyline = new GeneralPath();
    polyline.moveTo((float) xPoints[0], (float) yPoints[0]);
    
    for (int i = 1; i < xPoints.length; i++) {
      polyline.lineTo((float) xPoints[i], (float) yPoints[i]);
    };
    
    g2.draw(polyline);
  }
  
  /**
   * Gets the x coordinates of the underlying model defining the points to paint.
   */
  public DVector getModelX() {
    return x;
  }
  
  /**
   * Gets the y coordinates of the underlying model defining the points to paint.
   */
  public DVector getModelY() {
    return y;
  }
  
  /**
   * Sets underlying model defining the points to paint using a Point2dVector.<br>
   * Basically the model is defined by the x and y vectors.
   */
  public void setModel(Point2dVector model) {
    x = model.getXCoords();
    y = model.getYCoords();
    update();
  }
  
  /**
   * Sets underlying model defining the points to paint.
   * <P>The points are defined by two vectors of equal length, giving x and y
   * coordinates of all points.
   */
  public void setModel(DVector x1, DVector y1) {
    x = x1;
    y = y1;
    update();
  }
  
  /**
   * Sets all x values.<br>
   * This only leads to an update if fireOnX is true.
   */
  public void setXVector(DVector v) {
    x = v;
    
    // todo: clients use setModel(x, y) instead of two single calls
    // then: calling setXVector always updates!
    if (false) {
      update();
    }
  }
  
  /**
   * Sets all y values.<br>
   * This always leads to an update.
   */
  public void setYVector(DVector v) {
    y = v;
    update();
  }
  
  /**
   * Gets color of the line.
   */
  public Color getLineColor() {
    return this.lineColor;
  }
  
  /**
   * Sets color of the line.
   */
  public void setLineColor(Color lineColor) {
    this.lineColor = lineColor;
    update();
  }
}
