package physbeans.views;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.model.*;
import physbeans.inout.*;

/**
 * View that draws a color image representing a GridWithBoundary.
 * It is similar to ImageView, but directly uses the inner part of the grid
 * for the image buffer.
 * If the flag blowUp is true, it scales the image to the actual screen size.
 * <br>
 * It uses a color table with 256 entries to map scalar values to colors.
 */
public class GridView extends View implements Serializable {

  protected GridWithBoundary field;
  protected double minVal, maxVal;
  protected boolean infiniteScale;
  protected double scale;
  protected ColorTable colors;
  protected boolean blowUp, showBoundary;
  protected BufferedImage image;
  protected DataBufferByte buffer;
  protected AffineTransform gridTrafo;

  /** Creates a new instance of GridView */
  public GridView() {
    super();

    // create a standard colortable
    colors = new ColorTable();

    minVal = -1.0;
    maxVal = 1.0;
    infiniteScale = false;
    scale = 1.0;
    blowUp = true;
    showBoundary = false;
  }

  /**
   * calculates the image, using the known size
   */
  public void doLayout() {
    // get dimension of drawing area from panel
    Point dim = trafo.getMaxScreen();
    int size = dim.x * dim.y;
    if (size <= 0) {
      return;
    }

    createImage();       // create a new image with a new buffer size
    calcImageBuffer();   // calculate the values for the buffer from the field
    createGridTrafo(dim);
  }

  public void paintLayer(Graphics2D g2) {
    if (field != null) {
      if (blowUp) {
        // scale grid sized image buffer to component size
        Point dim = trafo.getMaxScreen();

        int nx = field.getGridSizeX();
        int ny = field.getGridSizeY();
        int bwX = field.getBoundaryWidthX();
        int bwY = field.getBoundaryWidthY();

        if (!showBoundary) {
          nx -= 2 * bwX;
          ny -= 2 * bwY;
        }
        Point nGrid = new Point(nx, ny);

        AffineTransform at = trafo.getInverseAffineTransform();
        at.scale(((double) dim.x) / nGrid.x, ((double) dim.y) / nGrid.y);
        g2.drawImage(image, at, myParent);

        if (showBoundary) {
          Rectangle2D boundary = new Rectangle2D.Double(
                  bwX, bwY, nx - 2 * bwX, ny - 2 * bwY);
          g2.draw(at.createTransformedShape(boundary));
        }
      } else {
        // use precomputed trafo to scale grid according to its world size
        g2.drawImage(image, gridTrafo, myParent);
      }

    }
  }

  /**
   * Gets scalar value that is mapped to highest color (255).
   */
  public double getMaxVal() {
    return maxVal;
  }

  /**
   * Sets scalar value that is mapped to highest color (255).<p>
   * Settings maxVal to Infinity leads to scaling with arctan
   */
  public void setMaxVal(double max) {
    maxVal = max;
    update();
  }

  /**
   * Gets scalar value that is mapped to lowest color (0).
   */
  public double getMinVal() {
    return minVal;
  }

  /**
   * Sets scalar value that is mapped to lowest color (0).
   */
  public void setMinVal(double min) {
    minVal = min;
    update();
  }

  /**
   * Gets scalar field to be displayed.
   */
  public GridWithBoundary getModel() {
    return this.field;
  }

  /**
   * Sets scalar field to be displayed.
   */
  public void setModel(GridWithBoundary model) {
    this.field = model;
    update();
  }

  /**
   * calculates the image buffer from the field data
   * faster version: uses only grid values.
   */
  protected void calcImageBuffer() {
    if (buffer == null || field == null) {
      return;
    }

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    int bx = field.getBoundaryWidthX();
    int by = field.getBoundaryWidthY();
    double[][] grid = getGrid();

    // scale down infinite range
    double val; // current field value
    int smin = showBoundary ? 0 : bx;
    int smax = showBoundary ? nx : nx - bx;
    int tmin = showBoundary ? 0 : by;
    int tmax = showBoundary ? ny : ny - by;

    if (infiniteScale) {
      for (int s = smin; s < smax; s++) {
        for (int t = tmin; t < tmax; t++) {
          // compress (-infinity .. infinity) to (-1 .. 1)
          val = 2 / Math.PI * Math.atan(grid[t][s] / scale);
          buffer.setElem((s - smin) + (smax - smin) * (t - tmin),
                  (byte) Math.round(255.0 * (val + 1.0) / 2.0));
        }
      }
    } else {
      for (int s = smin; s < smax; s++) {
        for (int t = tmin; t < tmax; t++) {
          val = Math.round(255.0 * (grid[t][s] - minVal) / (maxVal
                  - minVal));
          buffer.setElem((s - smin) + (smax - smin) * (t - tmin),
                  (byte) Math.max(0, Math.min(255, val)));
        }
      }
    }
  }

  protected double[][] getGrid() {
    // method is extracted so that ComplexGridView can override it.
    return field.getGrid();
  }

  /**
   * Creates an image with the size of the scalarfield's grid (without boundary layer).
   */
  protected void createImage() {
    if ((field == null) || (field.getGrid() == null)) {
      return;
    }

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    if (!showBoundary) {
      nx -= 2 * field.getBoundaryWidthX();
      ny -= 2 * field.getBoundaryWidthY();
    }

    image = new BufferedImage(nx, ny,
            BufferedImage.TYPE_BYTE_INDEXED, colors.getTable());
    WritableRaster raster = image.getRaster();
    buffer = (DataBufferByte) raster.getDataBuffer();
  }

  /**
   * Calculates trafo from grid to world.
   * This is done only once (at layout time) for performance reasons.
   */
  protected void createGridTrafo(Point dim) {
    if ((field == null) || (field.getGrid() == null)) {
      return;
    }

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    if (!showBoundary) {
      nx -= 2 * field.getBoundaryWidthX();
      ny -= 2 * field.getBoundaryWidthY();
    }

    Point nGrid = new Point(nx, ny);
    ScreenWorldTrafo swt = new ScreenWorldTrafo();
    swt.setMaxWorld(field.getMaxWorld());
    swt.setMinWorld(field.getMinWorld());
    swt.setMaxScreen(nGrid);

    gridTrafo = swt.getInverseAffineTransform();
  }

  /**
   * Recalculates and redraws the image using the current buffer.<br>
   */
  public void update() {
    Point dim = trafo.getMaxScreen();
    if (dim.x * dim.y <= 0) {
      return;
    }
    calcImageBuffer();
    super.update();
  }

  /**
   * Gets value of showBoundary flag.<br>
   * If true, the boundaries are displayed as well.
   */
  public boolean isShowBoundary() {
    return showBoundary;
  }

  /**
   * Sets value of showBoundary flag.<br>
   * If true, the boundaries are displayed as well.
   */
  public void setShowBoundary(boolean showBoundary) {
    this.showBoundary = showBoundary;
    update();
  }

  /**
   * Gets value of infiniteScale flag.<br>
   * If true, values are scaled using atan and a scale factor,
   * otherwise linearly between minVal and maxVal.
   */
  public boolean isInfiniteScale() {
    return this.infiniteScale;
  }

  /**
   * Sets value of infiniteScale flag.<br>
   * If true, values are scaled using atan and a scale factor,
   * otherwise linearly between minVal and maxVal.
   */
  public void setInfiniteScale(boolean infiniteScale) {
    this.infiniteScale = infiniteScale;
    update();
  }

  /**
   * Gets scale factor used with infinite scaling.
   */
  public double getScale() {
    return this.scale;
  }

  /**
   * Sets scale factor used with infinite scaling.
   */
  public void setScale(double scale) {
    this.scale = scale;
    update();
  }

  /**
   * Gets color table.
   */
  public ColorTable getColors() {
    return this.colors;
  }

  /**
   * Sets color table.
   */
  public void setColors(ColorTable colors) {
    this.colors = colors;
  }

  /**
   * Gets the blowUp flag.<br>
   * If true, the image is scaled to fill the LayeredScreen completely.
   */
  public boolean isBlowUp() {
    return blowUp;
  }

  /**
   * Sets the blowUp flag.<br>
   * If true, the image is scaled to fill the LayeredScreen completely.
   */
  public void setBlowUp(boolean b) {
    blowUp = b;
  }
}
