package physbeans.views;

import java.awt.*;

import physbeans.inout.ScreenWorldTrafo;
import physbeans.phys.*;

/**
 * View displaying an object and its rays. The object is symbolized by an arrow.
 */
public class ObjectLightSourceView extends View {
  
  protected LightSource model;
  protected LightRayBundleView raysView;
  protected ArrowView arrowView;
  protected boolean dashed;
  protected BasicStroke sDashed;
  
  /** Creates a new instance of ObjectLightSourceView */
  public ObjectLightSourceView() {
    raysView = new LightRayBundleView();
    arrowView = new ArrowView();
    setArrowWidth(0.3);
    
    dashed = false;
    float[] dash = {0.1f};
    sDashed = new BasicStroke(0.01f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER,
    1.0f, dash, 0.0f);
  }
  
  public void paintLayer(Graphics2D g2) {
    if (model == null) {
      return;  // do nothing
    }
    
    double x = model.getPosition();
    double y = model.getHeight();
    arrowView.setX(x);
    arrowView.setArrowLength(y);
    arrowView.setDirection(Math.PI/2);  // arrow is vertical
    arrowView.paintLayer(g2);
    
    raysView.setModel(model.getRays());
    Stroke oldStroke = g2.getStroke();
    if (dashed) {
      g2.setStroke(sDashed);
    }
    raysView.paintLayer(g2);
    g2.setStroke(oldStroke);
  }
  
  /**
   * Gets underlying model defining the light source to paint.
   */
  public LightSource getModel() {
    return model;
  }
  
  /**
   * Sets underlying model defining the light source to paint.
   */
  public void setModel(LightSource model) {
    this.model = model;
    update();
  }
  
  /**
   * Gets the color of the figure representing the source.
   */
  public Color getObjectColor() {
    return arrowView.getArrowColor();
  }
  
  /**
   * Sets the color of the figure representing the source.
   */
  public void setObjectColor(Color col) {
    arrowView.setArrowColor(col);
    update();
  }
  
  /**
   * Sets trafo between screen and world coordinates.
   */
  public void setTrafo(ScreenWorldTrafo trafo) {
    super.setTrafo(trafo);
    raysView.setTrafo(trafo);
    arrowView.setTrafo(trafo);
  }
  
  /**
   * Gets visibility status of all rays.
   * The four object rays are numbered 0 .. 3 in the following order: <p>
   *  parallel, focal, central, arbitrary.
   */
  public boolean[] getShowRays() {
    return raysView.getShowRays();
  }
  
  /**
   * Sets visibility status of all rays.
   * The four object rays are numbered 0 .. 3 in the following order: <p>
   *  parallel, focal, central, arbitrary.
   */
  public void setShowRays(boolean[] showRays) {
    raysView.setShowRays(showRays);
    update();
  }
  
  /**
   * Gets the dashed flag. If it is true, the rays are painted with dashed lines.
   */
  public boolean isDashed() {
    return this.dashed;
  }
  
  /**
   * Sets the dashed flag. If it is true, the rays are painted with dashed lines.
   */
  public void setDashed(boolean dashed) {
    this.dashed = dashed;
    update();
  }
  
  /**
   * Gets the width of the arrow.
   */
  public double getArrowWidth() {
    return arrowView.getArrowWidth();
  }
  
  /**
   * Sets the width of the arrow.
   */
  public void setArrowWidth(double w) {
    arrowView.setArrowWidth(w);
    update();
  }
  
  /**
   * Gets the color of the light rays.
   */
  public Color getRayColor() {
    return raysView.getLineColor();
  }
  
  /**
   * Sets the color of the light rays.
   */
  public void setRayColor(Color c) {
    raysView.setLineColor(c);
    update();
  }
}
