package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.math.*;

/**
 * Shows the cylinder of a Stirling motor with moving piston and recuperator.<br>
 * Input values are the gas volume and temperature and the phase (0, 1, 2, 3)
 * of the cycle.
 */
public class StirlingView extends CylinderView implements Serializable {
  
  protected double lReg;
  protected double TLow, THigh;
  
  /**
   * Creates a standard StirlingView.
   */
  public StirlingView() {
    super();
    
    lReg = 0.4;
    vMax = (lCyl - lReg)*Math.PI*dCyl*dCyl/4.0;
    
    setInitialValues(new DVector(0.5, 300.0, 0.0));
  }
  
  /**
   * Redraws the complete cylinder and piston according to driving parameters.
   */
  public void paintLayer(Graphics2D g) {
    double xP = model.get(0)*(lCyl - lReg)/vMax + lReg;
    double T = model.get(1);
    int phase = (int) Math.round(model.get(2));
    double xReg = getRegeneratorPosion(xP, T, phase);
    
    drawCylinder(g);
    drawPiston(g, xP);
    drawGas(g,xP, T);
    drawRegenerator(g, xReg);
    drawReservoirs(g);
  }
  
  /**
   * Sets phase of the Stirling cycle.
   * Conveniance function to simplify routing.
   */
  public void setPhase(double ph) {
    model.set(2, ph);
    update();
  }
  
  
  /**
   * Gets temperatuer of cold reservoir.
   */
  public double getTLow() {
    return TLow;
  }
  
  /**
   * Sets temperatuer of cold reservoir.
   */
  public void setTLow(double tLow) {
    this.TLow = tLow;
    update();
  }
  
  /**
   * Gets temperatuer of hot reservoir.
   */
  public double getTHigh() {
    return THigh;
  }
  
  /**
   * Sets temperatuer of hot reservoir.
   */
  public void setTHigh(double tHigh) {
    this.THigh = tHigh;
    update();
  }
  
  /**
   * Gets length of the regenerator.
   */
  public double getRegeneratorLength() {
    return this.lReg;
  }
  
  /**
   * Sets length of the regenerator.
   */
  public void setRegeneratorLength(double lReg) {
    this.lReg = lReg;
    update();
  }
  
  /**
   * Gets the position of (the right edge of) the regenerator.
   */
  protected double getRegeneratorPosion(double xP, double T, int phase) {
    double xR;
    
    switch (phase) {
      case 0:
        xR = lReg;   // regenerator is at the left end of the cylinder
        break;
      case 2:
        xR = xP;     // regenerator sits at the piston (right end of the cylinder)
      default:       // phases 1 and 3
        xR = (xP - lReg)*(T - TLow)/(THigh - TLow) + lReg;
    }
    
    return xR;
  }
  
  /**
   * Draws the regenerator at position xReg (= right edge).
   */
  protected void drawRegenerator(Graphics2D g, double xReg) {
    Rectangle2D rect = new Rectangle2D.Double(xReg - lReg, -dCyl/2.0, lReg, dCyl);
    g.draw(rect);
    
    // internal lines representing the flow through the regenerator
    double dt = 4*trafo.getScalingX();       // stripes of 3 pixel width
    int nPipes = 4;
    Line2D line1 = new Line2D.Double();
    Line2D line2 = new Line2D.Double();
    for (int i = 1; i <= nPipes; i++) {
      double t = -dCyl/2 + i*dCyl/(nPipes+1);
      line1.setLine(xReg - lReg, t - dt, xReg, t - dt);
      line2.setLine(xReg - lReg, t + dt, xReg, t + dt);
      g.draw(line1);
      g.draw(line2);
    }
  }
  
  /**
   * Draws the cold and hot heat reservoirs.
   */
  protected void drawReservoirs(Graphics2D g) {
    double xHot = 0.4*lCyl;
    double xCold = 0.5*lCyl;
    double h = 10*trafo.getScalingY();       // boxes of 10 pixel width
    
    // outer boxes
    Rectangle2D rect1 = new Rectangle2D.Double(0.0, -dCyl/2.0 - h, xHot, h);
    Rectangle2D rect2 = new Rectangle2D.Double(0.0, dCyl/2.0, xHot, h);
    Rectangle2D rect3 = new Rectangle2D.Double(xCold, -dCyl/2.0 - h, xHot, h);
    Rectangle2D rect4 = new Rectangle2D.Double(xCold, dCyl/2.0, xHot, h);
    
    // color the reservoirs according to their temperatures
    int col = (int) Math.round(scaler.evaluate(THigh));
    g.setColor(colorTable.getColor(col));
    rect1 = new Rectangle2D.Double(0.0, -dCyl/2.0 - h, xHot, h);
    g.fill(rect1);
    g.fill(rect2);
    
    col = (int) Math.round(scaler.evaluate(TLow));
    g.setColor(colorTable.getColor(col));
    g.fill(rect3);
    g.fill(rect4);
    
    // black borders around the boxes
    g.setColor(Color.BLACK);
    g.draw(rect1);
    g.draw(rect2);
    g.draw(rect3);
    g.draw(rect4);
  }
  
}
